@@ -137,6 +137,12 @@ static void waylandKdeHdrListener(void *data, FF_MAYBE_UNUSED struct kde_output_
     display->hdrEnabled = !!hdr_enabled;
 }
 
+static void waylandKdeMaxBitsPerColorListener(void *data, FF_MAYBE_UNUSED struct kde_output_device_v2 *kde_output_device_v2, uint32_t max_bpc)
+{
+    WaylandDisplay* display = data;
+    display->bitDepth = (uint8_t) max_bpc;
+}
+
 static struct kde_output_device_v2_listener outputListener = {
     .geometry = waylandKdeGeometryListener,
     .current_mode = waylandKdeCurrentModeListener,
@@ -163,13 +169,22 @@ static struct kde_output_device_v2_listener outputListener = {
     .sdr_gamut_wideness = (void*) stubListener,
     .color_profile_source = (void*) stubListener,
     .brightness = (void*) stubListener,
+    .color_power_tradeoff = (void*) stubListener,
+    .dimming = (void*) stubListener,
+    .replication_source = (void*) stubListener,
+    .ddc_ci_allowed = (void*) stubListener,
+    .max_bits_per_color = (void*) waylandKdeMaxBitsPerColorListener,
+    .max_bits_per_color_range = (void*) stubListener,
+    .automatic_max_bits_per_color_limit = (void*) stubListener,
+    .edr_policy = (void*) stubListener,
+    .sharpness = (void*) stubListener,
 };
 
-void ffWaylandHandleKdeOutput(WaylandData* wldata, struct wl_registry* registry, uint32_t name, uint32_t version)
+const char* ffWaylandHandleKdeOutput(WaylandData* wldata, struct wl_registry* registry, uint32_t name, uint32_t version)
 {
     struct wl_proxy* output = wldata->ffwl_proxy_marshal_constructor_versioned((struct wl_proxy*) registry, WL_REGISTRY_BIND, &kde_output_device_v2_interface, version, name, kde_output_device_v2_interface.name, version, NULL);
     if(output == NULL)
-        return;
+        return "Failed to create kde_output_device_v2";
 
     FF_LIST_AUTO_DESTROY modes = ffListCreate(sizeof(WaylandKdeMode));
     WaylandDisplay display = {
@@ -183,12 +198,21 @@ void ffWaylandHandleKdeOutput(WaylandData* wldata, struct wl_registry* registry,
         .internal = &modes,
     };
 
-    wldata->ffwl_proxy_add_listener(output, (void(**)(void)) &outputListener, &display);
-    wldata->ffwl_display_roundtrip(wldata->display);
+    if (wldata->ffwl_proxy_add_listener(output, (void(**)(void)) &outputListener, &display) < 0)
+    {
+        wldata->ffwl_proxy_destroy(output);
+        return "Failed to add listener to kde_output_device_v2";
+    }
+
+    if (wldata->ffwl_display_roundtrip(wldata->display) < 0)
+    {
+        wldata->ffwl_proxy_destroy(output);
+        return "Failed to roundtrip kde_output_device_v2";
+    }
     wldata->ffwl_proxy_destroy(output);
 
     if(display.width <= 0 || display.height <= 0 || !display.internal)
-        return;
+        return "Failed to get display information from kde_output_device_v2";
 
     uint32_t rotation = ffWaylandHandleRotation(&display);
 
@@ -226,11 +250,14 @@ void ffWaylandHandleKdeOutput(WaylandData* wldata, struct wl_registry* registry,
         item->manufactureYear = display.myear;
         item->manufactureWeek = display.mweek;
         item->serial = display.serial;
+        item->bitDepth = display.bitDepth;
     }
 
     ffStrbufDestroy(&display.description);
     ffStrbufDestroy(&display.name);
     ffStrbufDestroy(&display.edidName);
+
+    return NULL;
 }
 
 
@@ -241,20 +268,30 @@ static void waylandKdeOutputOrderListener(void *data, FF_MAYBE_UNUSED struct kde
         *id = ffWaylandGenerateIdFromName(output_name);
 }
 
-void ffWaylandHandleKdeOutputOrder(WaylandData* wldata, struct wl_registry* registry, uint32_t name, uint32_t version)
+const char* ffWaylandHandleKdeOutputOrder(WaylandData* wldata, struct wl_registry* registry, uint32_t name, uint32_t version)
 {
     struct wl_proxy* output = wldata->ffwl_proxy_marshal_constructor_versioned((struct wl_proxy*) registry, WL_REGISTRY_BIND, &kde_output_order_v1_interface, version, name, kde_output_order_v1_interface.name, version, NULL);
     if(output == NULL)
-        return;
+        return "Failed to create kde_output_order_v1";
 
     struct kde_output_order_v1_listener orderListener = {
         .output = waylandKdeOutputOrderListener,
         .done = (void*) stubListener,
     };
 
-    wldata->ffwl_proxy_add_listener(output, (void(**)(void)) &orderListener, &wldata->primaryDisplayId);
-    wldata->ffwl_display_roundtrip(wldata->display);
+    if (wldata->ffwl_proxy_add_listener(output, (void(**)(void)) &orderListener, &wldata->primaryDisplayId) < 0)
+    {
+        wldata->ffwl_proxy_destroy(output);
+        return "Failed to add listener to kde_output_order_v1";
+    }
+    if (wldata->ffwl_display_roundtrip(wldata->display) < 0)
+    {
+        wldata->ffwl_proxy_destroy(output);
+        return "Failed to roundtrip kde_output_order_v1";
+    }
     wldata->ffwl_proxy_destroy(output);
+
+    return NULL;
 }
 
 #endif
