@@ -1,6 +1,7 @@
 #pragma once
 
 #include "fastfetch.h"
+#include "modules/opengl/option.h"
 
 typedef struct FFOpenGLResult
 {
@@ -8,6 +9,10 @@ typedef struct FFOpenGLResult
     FFstrbuf renderer;
     FFstrbuf vendor;
     FFstrbuf slv;
+    FFstrbuf library;
 } FFOpenGLResult;
 
+#define FF_OPENGL_BUFFER_WIDTH 1
+#define FF_OPENGL_BUFFER_HEIGHT 1
+
 const char* ffDetectOpenGL(FFOpenGLOptions* options, FFOpenGLResult* result);
