@@ -10,6 +10,12 @@
 #include <stdlib.h>
 #include <assert.h>
 
+#ifdef FF_USE_SYSTEM_YYJSON
+    #include <yyjson.h>
+#else
+    #include "3rdparty/yyjson/yyjson.h"
+#endif
+
 #ifdef _WIN32
     // #include <shlwapi.h>
     __stdcall char* StrStrIA(const char* lpFirst, const char* lpSrch);
@@ -48,7 +54,6 @@ void ffStrbufPrependC(FFstrbuf* strbuf, char c);
 
 void ffStrbufInsertNC(FFstrbuf* strbuf, uint32_t index, uint32_t num, char c);
 
-void ffStrbufSetNS(FFstrbuf* strbuf, uint32_t length, const char* value);
 FF_C_PRINTF(2, 3) void ffStrbufSetF(FFstrbuf* strbuf, const char* format, ...);
 
 void ffStrbufTrimLeft(FFstrbuf* strbuf, char c);
@@ -73,6 +78,7 @@ bool ffStrbufSubstrAfter(FFstrbuf* strbuf, uint32_t index); // Not including the
 bool ffStrbufSubstrAfterFirstC(FFstrbuf* strbuf, char c);
 bool ffStrbufSubstrAfterFirstS(FFstrbuf* strbuf, const char* str);
 bool ffStrbufSubstrAfterLastC(FFstrbuf* strbuf, char c);
+bool ffStrbufSubstr(FFstrbuf* strbuf, uint32_t start, uint32_t end);
 
 FF_C_NODISCARD uint32_t ffStrbufCountC(const FFstrbuf* strbuf, char c);
 
@@ -83,7 +89,7 @@ bool ffStrbufEnsureEndsWithC(FFstrbuf* strbuf, char c);
 void ffStrbufWriteTo(const FFstrbuf* strbuf, FILE* file);
 void ffStrbufPutTo(const FFstrbuf* strbuf, FILE* file);
 
-FF_C_NODISCARD double ffStrbufToDouble(const FFstrbuf* strbuf);
+FF_C_NODISCARD double ffStrbufToDouble(const FFstrbuf* strbuf, double defaultValue);
 FF_C_NODISCARD int64_t ffStrbufToSInt(const FFstrbuf* strbuf, int64_t defaultValue);
 FF_C_NODISCARD uint64_t ffStrbufToUInt(const FFstrbuf* strbuf, uint64_t defaultValue);
 
@@ -95,6 +101,14 @@ void ffStrbufGetlineRestore(char** lineptr, size_t* n, FFstrbuf* buffer);
 bool ffStrbufRemoveDupWhitespaces(FFstrbuf* strbuf);
 bool ffStrbufMatchSeparatedNS(const FFstrbuf* strbuf, uint32_t compLength, const char* comp, char separator);
 
+int ffStrbufAppendUtf32CodePoint(FFstrbuf* strbuf, uint32_t codepoint);
+
+void ffStrbufAppendSInt(FFstrbuf* strbuf, int64_t value);
+void ffStrbufAppendUInt(FFstrbuf* strbuf, uint64_t value);
+// Appends a double value to the string buffer with the specified precision (0~15).
+// if `precision < 0`, let yyjson decide the precision
+void ffStrbufAppendDouble(FFstrbuf* strbuf, double value, int8_t precision);
+
 FF_C_NODISCARD static inline FFstrbuf ffStrbufCreateA(uint32_t allocate)
 {
     FFstrbuf strbuf;
@@ -208,6 +222,16 @@ static inline void ffStrbufAppendS(FFstrbuf* strbuf, const char* value)
     ffStrbufAppendNS(strbuf, (uint32_t) strlen(value), value);
 }
 
+static inline bool ffStrbufAppendJsonVal(FFstrbuf* strbuf, yyjson_val* jsonVal)
+{
+    if (yyjson_is_str(jsonVal))
+    {
+        ffStrbufAppendNS(strbuf, (uint32_t) unsafe_yyjson_get_len(jsonVal), unsafe_yyjson_get_str(jsonVal));
+        return true;
+    }
+    return false;
+}
+
 static inline void ffStrbufSetS(FFstrbuf* strbuf, const char* value)
 {
     ffStrbufClear(strbuf);
@@ -216,6 +240,12 @@ static inline void ffStrbufSetS(FFstrbuf* strbuf, const char* value)
         ffStrbufAppendNS(strbuf, (uint32_t) strlen(value), value);
 }
 
+static inline void ffStrbufSetNS(FFstrbuf* strbuf, uint32_t length, const char* value)
+{
+    ffStrbufClear(strbuf);
+    ffStrbufAppendNS(strbuf, length, value);
+}
+
 static inline void ffStrbufSet(FFstrbuf* strbuf, const FFstrbuf* value)
 {
     assert(value && value != strbuf);
@@ -227,6 +257,12 @@ static inline void ffStrbufSet(FFstrbuf* strbuf, const FFstrbuf* value)
     ffStrbufSetNS(strbuf, value->length, value->chars);
 }
 
+static inline bool ffStrbufSetJsonVal(FFstrbuf* strbuf, yyjson_val* jsonVal)
+{
+    ffStrbufClear(strbuf);
+    return ffStrbufAppendJsonVal(strbuf, jsonVal);
+}
+
 static inline void ffStrbufInit(FFstrbuf* strbuf)
 {
     extern char* CHAR_NULL_PTR;
@@ -281,6 +317,12 @@ FF_C_NODISCARD static inline FFstrbuf ffStrbufCreateNS(uint32_t length, const ch
     return strbuf;
 }
 
+static inline bool ffStrbufInitJsonVal(FFstrbuf* strbuf, yyjson_val* jsonVal)
+{
+    ffStrbufInit(strbuf);
+    return ffStrbufAppendJsonVal(strbuf, jsonVal);
+}
+
 static inline void ffStrbufInitS(FFstrbuf* strbuf, const char* str)
 {
     ffStrbufInit(strbuf);
